@@ -1,11 +1,10 @@
 import copy
+
 import numbers
 from functools import partial
 
 from bson import DBRef, ObjectId, SON, json_util
 import pymongo
-import six
-from six import iteritems
 
 from mongoengine import signals
 from mongoengine.base.common import get_document
@@ -25,14 +24,13 @@ from mongoengine.errors import (
     OperationError,
     ValidationError,
 )
-from mongoengine.python_support import Hashable
 
 __all__ = ("BaseDocument", "NON_FIELD_ERRORS")
 
 NON_FIELD_ERRORS = "__all__"
 
 
-class BaseDocument(object):
+class BaseDocument:
     # TODO simplify how `_changed_fields` is used.
     # Currently, handling of `_changed_fields` seems unnecessarily convoluted:
     # 1. `BaseDocument` defines `_changed_fields` in its `__slots__`, yet it's
@@ -95,7 +93,7 @@ class BaseDocument(object):
                 list(self._fields.keys()) + ["id", "pk", "_cls", "_text_score"]
             )
             if _undefined_fields:
-                msg = ('The fields "{0}" do not exist on the document "{1}"').format(
+                msg = ('The fields "{}" do not exist on the document "{}"').format(
                     _undefined_fields, self._class_name
                 )
                 raise FieldDoesNotExist(msg)
@@ -110,7 +108,7 @@ class BaseDocument(object):
         # Assign default values to the instance.
         # We set default values only for fields loaded from DB. See
         # https://github.com/mongoengine/mongoengine/issues/399 for more info.
-        for key, field in iteritems(self._fields):
+        for key, field in self._fields.items():
             if self._db_field_map.get(key, key) in __only_fields:
                 continue
             value = getattr(self, key, None)
@@ -122,14 +120,14 @@ class BaseDocument(object):
         # Set passed values after initialisation
         if self._dynamic:
             dynamic_data = {}
-            for key, value in iteritems(values):
+            for key, value in values.items():
                 if key in self._fields or key == "_id":
                     setattr(self, key, value)
                 else:
                     dynamic_data[key] = value
         else:
             FileField = _import_class("FileField")
-            for key, value in iteritems(values):
+            for key, value in values.items():
                 key = self._reverse_db_field_map.get(key, key)
                 if key in self._fields or key in ("id", "pk", "_cls"):
                     if __auto_convert and value is not None:
@@ -145,7 +143,7 @@ class BaseDocument(object):
 
         if self._dynamic:
             self._dynamic_lock = False
-            for key, value in iteritems(dynamic_data):
+            for key, value in dynamic_data.items():
                 setattr(self, key, value)
 
         # Flag initialised
@@ -163,7 +161,7 @@ class BaseDocument(object):
                 default = default()
             setattr(self, field_name, default)
         else:
-            super(BaseDocument, self).__delattr__(*args, **kwargs)
+            super().__delattr__(*args, **kwargs)
 
     def __setattr__(self, name, value):
         # Handle dynamic data only if an initialised dynamic document
@@ -210,9 +208,9 @@ class BaseDocument(object):
             and self__created
             and name == self._meta.get("id_field")
         ):
-            super(BaseDocument, self).__setattr__("_created", False)
+            super().__setattr__("_created", False)
 
-        super(BaseDocument, self).__setattr__(name, value)
+        super().__setattr__(name, value)
 
     def __getstate__(self):
         data = {}
@@ -248,7 +246,7 @@ class BaseDocument(object):
                 setattr(self, "_fields_ordered", _super_fields_ordered)
 
         dynamic_fields = data.get("_dynamic_fields") or SON()
-        for k in list(dynamic_fields.keys()):
+        for k in dynamic_fields.keys():
             setattr(self, k, data["_data"].get(k))
 
     def __iter__(self):
@@ -288,16 +286,13 @@ class BaseDocument(object):
         except (UnicodeEncodeError, UnicodeDecodeError):
             u = "[Bad Unicode data]"
         repr_type = str if u is None else type(u)
-        return repr_type("<%s: %s>" % (self.__class__.__name__, u))
+        return repr_type("<{}: {}>".format(self.__class__.__name__, u))
 
     def __str__(self):
         # TODO this could be simpler?
         if hasattr(self, "__unicode__"):
-            if six.PY3:
-                return self.__unicode__()
-            else:
-                return six.text_type(self).encode("utf-8")
-        return six.text_type("%s object" % self.__class__.__name__)
+            return self.__unicode__()
+        return "%s object" % self.__class__.__name__
 
     def __eq__(self, other):
         if (
@@ -446,7 +441,7 @@ class BaseDocument(object):
                 pk = self.pk
             elif self._instance and hasattr(self._instance, "pk"):
                 pk = self._instance.pk
-            message = "ValidationError (%s:%s) " % (self._class_name, pk)
+            message = "ValidationError ({}:{}) ".format(self._class_name, pk)
             raise ValidationError(message, errors=errors)
 
     def to_json(self, *args, **kwargs):
@@ -492,7 +487,7 @@ class BaseDocument(object):
             return cls(**value)
 
         if is_dict:
-            value = {k: self.__expand_dynamic_values(k, v) for k, v in list(value.items())}
+            value = {k: self.__expand_dynamic_values(k, v) for k, v in value.items()}
         else:
             value = [self.__expand_dynamic_values(name, v) for v in value]
 
@@ -519,7 +514,7 @@ class BaseDocument(object):
         if "." in key:
             key, rest = key.split(".", 1)
             key = self._db_field_map.get(key, key)
-            key = "%s.%s" % (key, rest)
+            key = "{}.{}".format(key, rest)
         else:
             key = self._db_field_map.get(key, key)
 
@@ -578,10 +573,10 @@ class BaseDocument(object):
         if not hasattr(data, "items"):
             iterator = enumerate(data)
         else:
-            iterator = iteritems(data)
+            iterator = data.items()
 
         for index_or_key, value in iterator:
-            item_key = "%s%s." % (base_key, index_or_key)
+            item_key = "{}{}.".format(base_key, index_or_key)
             # don't check anything lower if this key is already marked
             # as changed.
             if item_key[:-1] in changed_fields:
@@ -589,7 +584,7 @@ class BaseDocument(object):
 
             if hasattr(value, "_get_changed_fields"):
                 changed = value._get_changed_fields()
-                changed_fields += ["%s%s" % (item_key, k) for k in changed if k]
+                changed_fields += ["{}{}".format(item_key, k) for k in changed if k]
             elif isinstance(value, (list, tuple, dict)):
                 self._nestable_types_changed_fields(changed_fields, item_key, value)
 
@@ -620,7 +615,7 @@ class BaseDocument(object):
             if isinstance(data, EmbeddedDocument):
                 # Find all embedded fields that have been changed
                 changed = data._get_changed_fields()
-                changed_fields += ["%s%s" % (key, k) for k in changed if k]
+                changed_fields += ["{}{}".format(key, k) for k in changed if k]
             elif isinstance(data, (list, tuple, dict)):
                 if hasattr(field, "field") and isinstance(
                     field.field, (ReferenceField, GenericReferenceField)
@@ -744,7 +739,7 @@ class BaseDocument(object):
         # Convert SON to a data dict, making sure each key is a string and
         # corresponds to the right db field.
         data = {}
-        for key, value in iteritems(son):
+        for key, value in son.items():
             key = str(key)
             key = cls._db_field_map.get(key, key)
             data[key] = value
@@ -759,7 +754,7 @@ class BaseDocument(object):
         if not _auto_dereference:
             fields = copy.deepcopy(fields)
 
-        for field_name, field in iteritems(fields):
+        for field_name, field in fields.items():
             field._auto_dereference = _auto_dereference
             if field.db_field in data:
                 value = data[field.db_field]
@@ -774,17 +769,16 @@ class BaseDocument(object):
 
         if errors_dict:
             errors = "\n".join(
-                ["Field '%s' - %s" % (k, v) for k, v in list(errors_dict.items())]
+                ["Field '{}' - {}".format(k, v) for k, v in errors_dict.items()]
             )
-            msg = "Invalid data to create a `%s` instance.\n%s" % (
-                cls._class_name,
-                errors,
+            msg = "Invalid data to create a `{}` instance.\n{}".format(
+                cls._class_name, errors,
             )
             raise InvalidDocumentError(msg)
 
         # In STRICT documents, remove any keys that aren't in cls._fields
         if cls.STRICT:
-            data = {k: v for k, v in iteritems(data) if k in cls._fields}
+            data = {k: v for k, v in data.items() if k in cls._fields}
 
         obj = cls(
             __auto_convert=False, _created=created, __only_fields=only_fields, **data
@@ -831,7 +825,7 @@ class BaseDocument(object):
     @classmethod
     def _build_index_spec(cls, spec):
         """Build a PyMongo index spec from a MongoEngine index spec."""
-        if isinstance(spec, six.string_types):
+        if isinstance(spec, str):
             spec = {"fields": [spec]}
         elif isinstance(spec, (list, tuple)):
             spec = {"fields": list(spec)}
@@ -919,7 +913,7 @@ class BaseDocument(object):
     def _unique_with_indexes(cls, namespace=""):
         """Find unique indexes in the document schema and return them."""
         unique_indexes = []
-        for field_name, field in list(cls._fields.items()):
+        for field_name, field in cls._fields.items():
             sparse = field.sparse
 
             # Generate a list of indexes needed by uniqueness constraints
@@ -928,7 +922,7 @@ class BaseDocument(object):
 
                 # Add any unique_with fields to the back of the index spec
                 if field.unique_with:
-                    if isinstance(field.unique_with, six.string_types):
+                    if isinstance(field.unique_with, str):
                         field.unique_with = [field.unique_with]
 
                     # Convert unique_with field names to real field names
@@ -949,7 +943,8 @@ class BaseDocument(object):
 
                 # Add the new index to the list
                 fields = [
-                    ("%s%s" % (namespace, f), pymongo.ASCENDING) for f in unique_fields
+                    ("{}{}".format(namespace, f), pymongo.ASCENDING)
+                    for f in unique_fields
                 ]
                 index = {"fields": fields, "unique": True, "sparse": sparse}
                 unique_indexes.append(index)
@@ -990,7 +985,7 @@ class BaseDocument(object):
             [_import_class(field) for field in geo_field_type_names]
         )
 
-        for field in list(cls._fields.values()):
+        for field in cls._fields.values():
             if not isinstance(field, geo_field_types):
                 continue
 
@@ -1006,7 +1001,7 @@ class BaseDocument(object):
             elif field._geo_index:
                 field_name = field.db_field
                 if parent_field:
-                    field_name = "%s.%s" % (parent_field, field_name)
+                    field_name = "{}.{}".format(parent_field, field_name)
                 geo_indices.append({"fields": [(field_name, field._geo_index)]})
 
         return geo_indices
@@ -1154,7 +1149,7 @@ class BaseDocument(object):
         """For each field that specifies choices, create a
         get_<field>_display method.
         """
-        fields_with_choices = [(n, f) for n, f in list(self._fields.items()) if f.choices]
+        fields_with_choices = [(n, f) for n, f in self._fields.items() if f.choices]
         for attr_name, field in fields_with_choices:
             setattr(
                 self,
@@ -1175,9 +1170,6 @@ class BaseDocument(object):
                 else [value]
             )
             return sep.join(
-                [
-                    six.text_type(dict(field.choices).get(val, val))
-                    for val in values or []
-                ]
+                [str(dict(field.choices).get(val, val)) for val in values or []]
             )
         return value
