@@ -7,7 +7,6 @@ _PYMONGO_37 = (3, 7)
 
 PYMONGO_VERSION = tuple(pymongo.version_tuple[:2])
 
-IS_PYMONGO_3 = PYMONGO_VERSION[0] >= 3
 IS_PYMONGO_GTE_37 = PYMONGO_VERSION >= _PYMONGO_37
 
 
@@ -28,6 +27,6 @@ def list_collection_names(db, include_system_collections=False):
         collections = db.collection_names()
 
     if not include_system_collections:
-        collections = [c for c in collections if not c.startswith('system.')]
+        collections = [c for c in collections if not c.startswith("system.")]
 
     return collections
